@@ -19,21 +19,21 @@ EXTREME_TIMEOUT = 30
 
 # If True, existing logs from past test runs will be saved and take up space.
 # If False, only the logs from the most recent test run will be saved locally.
+# You can also archive existing logs on the command line with: "--archive_logs"
 ARCHIVE_EXISTING_LOGS = False
 
 # If True, existing downloads from past runs will be saved and take up space.
 # If False, only the downloads from the most recent run will be saved locally.
 ARCHIVE_EXISTING_DOWNLOADS = False
 
-# Default names for files saved during test failures when logging is turned on.
-# (These files will get saved to the "latest_logs/" folder)
-# Usage: "--with-testing_base"
+# Default names for files saved during test failures.
+# (These files will get saved to the "latest_logs/" folder.)
 SCREENSHOT_NAME = "screenshot.png"
 BASIC_INFO_NAME = "basic_test_info.txt"
 PAGE_SOURCE_NAME = "page_source.html"
 
-# Default names for folders and files saved when reports are turned on.
-# Usage: "--report" and "--with-testing_base" together. (NOSETESTS only)
+# Default names for files and folders saved when using nosetests reports.
+# Usage: "--report". (NOSETESTS only)
 LATEST_REPORT_DIR = "latest_report"
 REPORT_ARCHIVE_DIR = "archived_reports"
 HTML_REPORT = "report.html"
@@ -54,9 +54,6 @@ This adds wait_for_angularjs() after various browser actions.
 '''
 WAIT_FOR_ANGULARJS = True
 
-# Option to start Chrome in full screen mode by default
-START_CHROME_IN_FULL_SCREEN_MODE = False
-
 # Default time to wait after each browser action performed during Demo Mode.
 # Use Demo Mode when you want others to see what your automation is doing.
 # Usage: "--demo_mode". (Can be overwritten by using "--demo_sleep=TIME".)
@@ -70,6 +67,32 @@ HIGHLIGHTS = 4
 # Messenger notifications appear when reaching assert statements in Demo Mode.
 DEFAULT_MESSAGE_DURATION = 2.55
 
+# If True, the Content Security Policy will be disabled on Firefox.
+# If False, each website's default Content Security Policy will be used.
+# (A website's CSP may prevent SeleniumBase from loading custom JavaScript.)
+# If using demo_mode or MasterQA, this value will become True regardless.
+# You can also disable the CSP on the command line by using "--disable_csp".
+DISABLE_CSP_ON_FIREFOX = True
+
+# If True, the Content Security Policy will be disabled on Chrome.
+# If False, each website's default Content Security Policy will be used.
+# (A website's CSP may prevent SeleniumBase from loading custom JavaScript.)
+# You can also disable the CSP on the command line by using "--disable_csp".
+DISABLE_CSP_ON_CHROME = False
+
+# If True, an Exception is raised immediately for invalid proxy string syntax.
+# If False, a Warning will appear after the test, with no proxy server used.
+# (This applies when using --proxy=[PROXY_STRING] for using a proxy server.)
+RAISE_INVALID_PROXY_STRING_EXCEPTION = True
+
+# Default browser resolutions when opening new windows for tests.
+# (Headless resolutions take priority, and include all browsers.)
+# (Firefox starts maximized by default when running in GUI Mode.)
+CHROME_START_WIDTH = 1250
+CHROME_START_HEIGHT = 840
+HEADLESS_START_WIDTH = 1440
+HEADLESS_START_HEIGHT = 1880
+
 # #####>>>>>----- MasterQA SETTINGS -----<<<<<#####
 # ##### (Used when importing MasterQA as the parent class)
 
@@ -98,8 +121,10 @@ TOTP_KEY = "base32secretABCD"
 
 
 # MySQL DB Credentials
-# (For saving data from tests)
+# (For saving data from tests to a MySQL DB)
+# Usage: "--with-db_reporting"
 DB_HOST = "127.0.0.1"
+DB_PORT = 3306
 DB_USERNAME = "root"
 DB_PASSWORD = "test"
 DB_SCHEMA = "test_db"
@@ -108,6 +133,7 @@ DB_SCHEMA = "test_db"
 # Amazon S3 Bucket Credentials
 # (For saving screenshots and other log files from tests)
 # (Bucket names are unique across all existing bucket names in Amazon S3)
+# Usage: "--with-s3_logging"
 S3_LOG_BUCKET = "[S3 BUCKET NAME]"
 S3_BUCKET_URL = "https://s3.amazonaws.com/[S3 BUCKET NAME]/"
 S3_SELENIUM_ACCESS_KEY = "[S3 ACCESS KEY]"
@@ -122,21 +148,3 @@ ENCRYPTION_KEY = "Pg^.l!8UdJ+Y7dMIe&fl*%!p9@ej]/#tL~3E4%6?"
 # Helps identify which strings/passwords have been obfuscated.
 OBFUSCATION_START_TOKEN = "$^*ENCRYPT="
 OBFUSCATION_END_TOKEN = "?&#$"
-
-
-# #####>>>>>----- OPTIONAL SETTINGS -----<<<<<#####
-# ##### (For reading emails, notifying people via chat apps, etc.)
-
-# Default Email Credentials
-# (If tests send out emails, you can scan and verify them by using IMAP)
-# Here's a list of imap strings for known email providers:
-# - Gmail:         imap.gmail.com
-# - Outlook/Live:  imap-mail.outlook.com
-# - Yahoo Mail:    imap.mail.yahoo.com
-# - AT&T:          imap.mail.att.net
-# - Comcast:       imap.comcast.net
-# - Verizon:       incoming.verizon.net
-EMAIL_USERNAME = "[TEST ACCOUNT GMAIL USERNAME]@gmail.com"
-EMAIL_PASSWORD = "[TEST ACCOUNT GMAIL PASSWORD]"
-EMAIL_IMAP_STRING = "imap.gmail.com"
-EMAIL_IMAP_PORT = 993
