@@ -102,7 +102,7 @@ bt_backdrop_style = (
     '''
     .tour-tour-element {
         box-shadow: 0 0 0 99999px rgba(0, 0, 0, 0.20);
-        pointer-events:  none !important;
+        pointer-events: none !important;
         z-index: 9999;
     }
 
@@ -111,11 +111,19 @@ bt_backdrop_style = (
     }
 
     .tour-tour {
-        pointer-events:  auto;
+        pointer-events: auto;
         z-index: 9999;
     }
     ''')
 
+messenger_style = (
+    '''
+    .messenger-message-inner {
+        font-family: "Proxima Nova","proxima-nova",Arial,sans-serif !important;
+        font-size: 17px;
+    }
+    ''')
+
 sh_style_test = (
     '''
     var test_tour = new Shepherd.Tour({
@@ -139,13 +147,13 @@ sh_backdrop_style = (
     '''
     body.shepherd-active .shepherd-target.shepherd-enabled {
         box-shadow: 0 0 0 99999px rgba(0, 0, 0, 0.20);
-        pointer-events:  none !important;
+        pointer-events: none !important;
         z-index: 9999;
     }
 
     body.shepherd-active .shepherd-orphan {
         box-shadow: 0 0 0 99999px rgba(0, 0, 0, 0.20);
-        pointer-events:  auto;
+        pointer-events: auto;
         z-index: 9999;
     }
 
@@ -160,11 +168,11 @@ sh_backdrop_style = (
     }
 
     body.shepherd-active .shepherd-step {
-        pointer-events:  auto;
+        pointer-events: auto;
         z-index: 9999;
     }
 
     body.shepherd-active {
-        pointer-events:  none !important;
+        pointer-events: none !important;
     }
     ''')
