@@ -6,8 +6,12 @@ import re
 import requests
 import time
 from selenium.common.exceptions import WebDriverException
+from seleniumbase import config as sb_config
+from seleniumbase.common import decorators
 from seleniumbase.config import settings
+from seleniumbase.core import style_sheet
 from seleniumbase.fixtures import constants
+from seleniumbase.fixtures import shared_utils
 
 
 def wait_for_ready_state_complete(driver, timeout=settings.EXTREME_TIMEOUT):
@@ -17,10 +21,10 @@ def wait_for_ready_state_complete(driver, timeout=settings.EXTREME_TIMEOUT):
     fully loaded (although AJAX and other loads might still be happening).
     This method will wait until document.readyState == "complete".
     """
-
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             ready_state = driver.execute_script("return document.readyState")
         except WebDriverException:
@@ -73,6 +77,14 @@ def wait_for_angularjs(driver, timeout=settings.LARGE_TIMEOUT, **kwargs):
         time.sleep(0.05)
 
 
+def is_html_inspector_activated(driver):
+    try:
+        driver.execute_script("HTMLInspector")  # Fails if not defined
+        return True
+    except Exception:
+        return False
+
+
 def is_jquery_activated(driver):
     try:
         driver.execute_script("jQuery('html')")  # Fails if jq is not defined
@@ -97,6 +109,15 @@ def wait_for_jquery_active(driver, timeout=None):
             time.sleep(0.1)
 
 
+def raise_unable_to_load_jquery_exception(driver):
+    """ The most-likely reason for jQuery not loading on web pages. """
+    raise Exception(
+        '''Unable to load jQuery on "%s" due to a possible violation '''
+        '''of the website's Content Security Policy directive. '''
+        '''To override this policy, add "--disable-csp" on the '''
+        '''command-line when running your tests.''' % driver.current_url)
+
+
 def activate_jquery(driver):
     """ If "jQuery is not defined", use this method to activate it for use.
         This happens because jQuery is not always defined on web sites. """
@@ -122,10 +143,7 @@ def activate_jquery(driver):
         except Exception:
             time.sleep(0.1)
     # Since jQuery still isn't activating, give up and raise an exception
-    raise Exception(
-        '''Unable to load jQuery on "%s" due to a possible violation '''
-        '''of the website's Content Security Policy '''
-        '''directive. ''' % driver.current_url)
+    raise_unable_to_load_jquery_exception(driver)
 
 
 def are_quotes_escaped(string):
@@ -198,43 +216,43 @@ def wait_for_css_query_selector(
 
 
 def highlight_with_js(driver, selector, loops, o_bs):
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: 0px 0px 6px 6px rgba(128, 128, 128, 0.5)';"""
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '0px 0px 6px 6px rgba(128, 128, 128, 0.5)';"""
               % selector)
     driver.execute_script(script)
     for n in range(loops):
-        script = ("""document.querySelector('%s').style =
-                  'box-shadow: 0px 0px 6px 6px rgba(255, 0, 0, 1)';"""
+        script = ("""document.querySelector('%s').style.boxShadow =
+                  '0px 0px 6px 6px rgba(255, 0, 0, 1)';"""
                   % selector)
         driver.execute_script(script)
         time.sleep(0.0181)
-        script = ("""document.querySelector('%s').style =
-                  'box-shadow: 0px 0px 6px 6px rgba(128, 0, 128, 1)';"""
+        script = ("""document.querySelector('%s').style.boxShadow =
+                  '0px 0px 6px 6px rgba(128, 0, 128, 1)';"""
                   % selector)
         driver.execute_script(script)
         time.sleep(0.0181)
-        script = ("""document.querySelector('%s').style =
-                  'box-shadow: 0px 0px 6px 6px rgba(0, 0, 255, 1)';"""
+        script = ("""document.querySelector('%s').style.boxShadow =
+                  '0px 0px 6px 6px rgba(0, 0, 255, 1)';"""
                   % selector)
         driver.execute_script(script)
         time.sleep(0.0181)
-        script = ("""document.querySelector('%s').style =
-                  'box-shadow: 0px 0px 6px 6px rgba(0, 255, 0, 1)';"""
+        script = ("""document.querySelector('%s').style.boxShadow =
+                  '0px 0px 6px 6px rgba(0, 255, 0, 1)';"""
                   % selector)
         driver.execute_script(script)
         time.sleep(0.0181)
-        script = ("""document.querySelector('%s').style =
-                  'box-shadow: 0px 0px 6px 6px rgba(128, 128, 0, 1)';"""
+        script = ("""document.querySelector('%s').style.boxShadow =
+                  '0px 0px 6px 6px rgba(128, 128, 0, 1)';"""
                   % selector)
         driver.execute_script(script)
         time.sleep(0.0181)
-        script = ("""document.querySelector('%s').style =
-                  'box-shadow: 0px 0px 6px 6px rgba(128, 0, 128, 1)';"""
+        script = ("""document.querySelector('%s').style.boxShadow =
+                  '0px 0px 6px 6px rgba(128, 0, 128, 1)';"""
                   % selector)
         driver.execute_script(script)
         time.sleep(0.0181)
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: %s';"""
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '%s';"""
               % (selector, o_bs))
     driver.execute_script(script)
 
@@ -291,14 +309,14 @@ def add_css_link(driver, css_link):
 def add_js_link(driver, js_link):
     script_to_add_js = (
         """function injectJS(link) {
-              var head = document.getElementsByTagName("head")[0];
+              var body = document.getElementsByTagName("body")[0];
               var script = document.createElement("script");
               script.src = link;
               script.defer;
               script.type="text/javascript";
               script.crossorigin = "anonymous";
               script.onload = function() { null };
-              head.appendChild(script);
+              body.appendChild(script);
            }
            injectJS("%s");""")
     js_link = escape_quotes_if_needed(js_link)
@@ -325,13 +343,26 @@ def add_js_code_from_link(driver, js_link):
         js_link = "http:" + js_link
     js_code = requests.get(js_link).text
     add_js_code_script = (
-        '''var h = document.getElementsByTagName('head').item(0);'''
-        '''var s = document.createElement("script");'''
-        '''s.type = "text/javascript";'''
-        '''s.onload = function() { null };'''
-        '''s.appendChild(document.createTextNode("%s"));'''
-        '''h.appendChild(s);''')
-    js_code = js_code.replace('\n', '')
+        '''var body = document.getElementsByTagName('body').item(0);'''
+        '''var script = document.createElement("script");'''
+        '''script.type = "text/javascript";'''
+        '''script.onload = function() { null };'''
+        '''script.appendChild(document.createTextNode("%s"));'''
+        '''body.appendChild(script);''')
+    js_code = js_code.replace('\n', ' ')
+    js_code = escape_quotes_if_needed(js_code)
+    driver.execute_script(add_js_code_script % js_code)
+
+
+def add_js_code(driver, js_code):
+    add_js_code_script = (
+        '''var body = document.getElementsByTagName('body').item(0);'''
+        '''var script = document.createElement("script");'''
+        '''script.type = "text/javascript";'''
+        '''script.onload = function() { null };'''
+        '''script.appendChild(document.createTextNode("%s"));'''
+        '''body.appendChild(script);''')
+    js_code = js_code.replace('\n', ' ')
     js_code = escape_quotes_if_needed(js_code)
     driver.execute_script(add_js_code_script % js_code)
 
@@ -368,11 +399,11 @@ def activate_jquery_confirm(driver):
 
     if not is_jquery_activated(driver):
         add_js_link(driver, jquery_js)
-        wait_for_jquery_active(driver, timeout=0.6)
+        wait_for_jquery_active(driver, timeout=0.9)
     add_css_link(driver, jq_confirm_css)
     add_js_link(driver, jq_confirm_js)
 
-    for x in range(7):
+    for x in range(15):
         # jQuery-Confirm needs a small amount of time to load & activate.
         try:
             driver.execute_script("jconfirm")
@@ -383,6 +414,28 @@ def activate_jquery_confirm(driver):
             time.sleep(0.1)
 
 
+def activate_html_inspector(driver):
+    jquery_js = constants.JQuery.MIN_JS
+    html_inspector_js = constants.HtmlInspector.MIN_JS
+
+    if is_html_inspector_activated(driver):
+        return
+    if not is_jquery_activated(driver):
+        add_js_link(driver, jquery_js)
+        wait_for_jquery_active(driver, timeout=0.9)
+    add_js_link(driver, html_inspector_js)
+
+    for x in range(15):
+        # HTML-Inspector needs a small amount of time to load & activate.
+        try:
+            driver.execute_script("HTMLInspector")
+            wait_for_ready_state_complete(driver)
+            wait_for_angularjs(driver)
+            return
+        except Exception:
+            time.sleep(0.1)
+
+
 def activate_messenger(driver):
     jquery_js = constants.JQuery.MIN_JS
     messenger_css = constants.Messenger.MIN_CSS
@@ -404,6 +457,7 @@ def activate_messenger(driver):
                  "theme: 'future'}")
 
     add_js_link(driver, jquery_js)
+    wait_for_jquery_active(driver, timeout=0.2)
     add_css_link(driver, messenger_css)
     add_css_link(driver, msgr_theme_flat_css)
     add_css_link(driver, msgr_theme_future_css)
@@ -416,6 +470,7 @@ def activate_messenger(driver):
     add_js_link(driver, messenger_js)
     add_js_link(driver, msgr_theme_flat_js)
     add_js_link(driver, msgr_theme_future_js)
+    add_css_style(driver, style_sheet.messenger_style)
 
     for x in range(int(settings.MINI_TIMEOUT * 10.0)):
         # Messenger needs a small amount of time to load & activate.
@@ -470,19 +525,17 @@ def set_messenger_theme(driver, theme="default", location="default",
     time.sleep(0.1)
 
 
-def post_message(driver, message, msg_dur, style="info", duration=None):
+def post_message(driver, message, msg_dur, style="info"):
     """ A helper method to post a message on the screen with Messenger.
         (Should only be called from post_message() in base_case.py) """
-    if not duration:
-        if not msg_dur:
-            duration = settings.DEFAULT_MESSAGE_DURATION
-        else:
-            duration = msg_dur
+    if not msg_dur:
+        msg_dur = settings.DEFAULT_MESSAGE_DURATION
+    msg_dur = float(msg_dur)
     message = re.escape(message)
     message = escape_quotes_if_needed(message)
     messenger_script = ('''Messenger().post({message: "%s", type: "%s", '''
                         '''hideAfter: %s, hideOnNavigate: true});'''
-                        % (message, style, duration))
+                        % (message, style, msg_dur))
     try:
         driver.execute_script(messenger_script)
     except Exception:
@@ -499,71 +552,76 @@ def post_message(driver, message, msg_dur, style="info", duration=None):
             driver.execute_script(messenger_script)
 
 
-def post_messenger_success_message(driver, message, msg_dur, duration=None):
-    if not duration:
-        if not msg_dur:
-            duration = settings.DEFAULT_MESSAGE_DURATION
-        else:
-            duration = msg_dur
+def post_messenger_success_message(driver, message, msg_dur):
+    if not msg_dur:
+        msg_dur = settings.DEFAULT_MESSAGE_DURATION
+    msg_dur = float(msg_dur)
     try:
-        set_messenger_theme(driver, theme="future", location="bottom_right")
+        theme = "future"
+        location = "bottom_right"
+        if sb_config.mobile_emulator:
+            theme = "block"
+            location = "top_center"
+        set_messenger_theme(driver, theme=theme, location=location)
         post_message(
-            driver, message, msg_dur, style="success", duration=duration)
-        time.sleep(duration)
+            driver, message, msg_dur, style="success")
+        time.sleep(msg_dur + 0.07)
     except Exception:
         pass
 
 
-def post_messenger_error_message(driver, message, msg_dur, duration=None):
-    if not duration:
-        if not msg_dur:
-            duration = settings.DEFAULT_MESSAGE_DURATION
-        else:
-            duration = msg_dur
+def post_messenger_error_message(driver, message, msg_dur):
+    if not msg_dur:
+        msg_dur = settings.DEFAULT_MESSAGE_DURATION
+    msg_dur = float(msg_dur)
     try:
         set_messenger_theme(driver, theme="block", location="top_center")
         post_message(
-            driver, message, msg_dur, style="error", duration=duration)
-        time.sleep(duration)
+            driver, message, msg_dur, style="error")
+        time.sleep(msg_dur + 0.07)
     except Exception:
         pass
 
 
 def highlight_with_js_2(driver, message, selector, o_bs, msg_dur):
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: 0px 0px 6px 6px rgba(128, 128, 128, 0.5)';"""
+    if selector == "html":
+        selector = "body"
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '0px 0px 6px 6px rgba(128, 128, 128, 0.5)';"""
               % selector)
     driver.execute_script(script)
     time.sleep(0.0181)
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: 0px 0px 6px 6px rgba(205, 30, 0, 1)';"""
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '0px 0px 6px 6px rgba(205, 30, 0, 1)';"""
               % selector)
     driver.execute_script(script)
     time.sleep(0.0181)
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: 0px 0px 6px 6px rgba(128, 0, 128, 1)';"""
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '0px 0px 6px 6px rgba(128, 0, 128, 1)';"""
               % selector)
     driver.execute_script(script)
     time.sleep(0.0181)
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: 0px 0px 6px 6px rgba(50, 50, 128, 1)';"""
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '0px 0px 6px 6px rgba(50, 50, 128, 1)';"""
               % selector)
     driver.execute_script(script)
     time.sleep(0.0181)
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: 0px 0px 6px 6px rgba(50, 205, 50, 1)';"""
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '0px 0px 6px 6px rgba(50, 205, 50, 1)';"""
               % selector)
     driver.execute_script(script)
     time.sleep(0.0181)
 
     post_messenger_success_message(driver, message, msg_dur)
 
-    script = ("""document.querySelector('%s').style =
-              'box-shadow: %s';""" % (selector, o_bs))
+    script = ("""document.querySelector('%s').style.boxShadow =
+              '%s';""" % (selector, o_bs))
     driver.execute_script(script)
 
 
 def highlight_with_jquery_2(driver, message, selector, o_bs, msg_dur):
+    if selector == "html":
+        selector = "body"
     script = """jQuery('%s').css('box-shadow',
         '0px 0px 6px 6px rgba(128, 128, 128, 0.5)');""" % selector
     safe_execute_script(driver, script)
@@ -592,7 +650,12 @@ def highlight_with_jquery_2(driver, message, selector, o_bs, msg_dur):
 
 
 def scroll_to_element(driver, element):
-    element_location = element.location['y']
+    element_location = None
+    try:
+        element_location = element.location['y']
+    except Exception:
+        # element.location_once_scrolled_into_view  # Old hack
+        return False
     element_location = element_location - 130
     if element_location < 0:
         element_location = 0
@@ -601,8 +664,9 @@ def scroll_to_element(driver, element):
     # scroll_script = "jQuery('%s')[0].scrollIntoView()" % selector
     try:
         driver.execute_script(scroll_script)
-    except WebDriverException:
-        pass  # Older versions of Firefox experienced issues here
+        return True
+    except Exception:
+        return False
 
 
 def slow_scroll_to_element(driver, element, browser):
@@ -611,7 +675,12 @@ def slow_scroll_to_element(driver, element, browser):
         scroll_to_element(driver, element)
         return
     scroll_position = driver.execute_script("return window.scrollY;")
-    element_location = element.location['y']
+    element_location = None
+    try:
+        element_location = element.location['y']
+    except Exception:
+        element.location_once_scrolled_into_view
+        return
     element_location = element_location - 130
     if element_location < 0:
         element_location = 0
@@ -634,6 +703,7 @@ def slow_scroll_to_element(driver, element, browser):
         time.sleep(0.162)
 
 
+@decorators.deprecated("Use re.escape() instead, which does what you want!")
 def _jq_format(code):
     """
     DEPRECATED - Use re.escape() instead, which performs the intended action.
