@@ -24,6 +24,8 @@ import os
 import sys
 import time
 import traceback
+from selenium.common.exceptions import StaleElementReferenceException
+from selenium.common import exceptions as selenium_exceptions
 from selenium.webdriver.common.by import By
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.remote.errorhandler import ElementNotVisibleException
@@ -32,16 +34,19 @@ from selenium.webdriver.remote.errorhandler import NoAlertPresentException
 from selenium.webdriver.remote.errorhandler import NoSuchFrameException
 from selenium.webdriver.remote.errorhandler import NoSuchWindowException
 from seleniumbase.config import settings
+from seleniumbase.core import log_helper
+from seleniumbase.fixtures import page_utils
+from seleniumbase.fixtures import shared_utils
+ENI_Exception = selenium_exceptions.ElementNotInteractableException
 
 
 def is_element_present(driver, selector, by=By.CSS_SELECTOR):
     """
-    Searches for the specified element by the given selector.  Returns whether
-    the element object if the element is present on the page.
+    Returns whether the specified element selector is present on the page.
     @Params
     driver - the webdriver object (required)
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     @Returns
     Boolean (is element present)
     """
@@ -54,12 +59,11 @@ def is_element_present(driver, selector, by=By.CSS_SELECTOR):
 
 def is_element_visible(driver, selector, by=By.CSS_SELECTOR):
     """
-    Searches for the specified element by the given selector.  Returns whether
-    the element object if the element is present and visible on the page.
+    Returns whether the specified element selector is visible on the page.
     @Params
     driver - the webdriver object (required)
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     @Returns
     Boolean (is element visible)
     """
@@ -72,14 +76,12 @@ def is_element_visible(driver, selector, by=By.CSS_SELECTOR):
 
 def is_text_visible(driver, text, selector, by=By.CSS_SELECTOR):
     """
-    Searches for the specified element by the given selector. Returns whether
-    the element object if the element is present and visible on the page and
-    contains the given text.
+    Returns whether the specified text is visible in the specified selector.
     @Params
     driver - the webdriver object (required)
     text - the text string to search for
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     @Returns
     Boolean (is text visible)
     """
@@ -95,14 +97,22 @@ def hover_on_element(driver, selector, by=By.CSS_SELECTOR):
     Fires the hover event for the specified element by the given selector.
     @Params
     driver - the webdriver object (required)
-    selector - the locator (css selector) that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     """
     element = driver.find_element(by=by, value=selector)
     hover = ActionChains(driver).move_to_element(element)
     hover.perform()
 
 
+def hover_element(driver, element):
+    """
+    Similar to hover_on_element(), but uses found element, not a selector.
+    """
+    hover = ActionChains(driver).move_to_element(element)
+    hover.perform()
+
+
 def hover_and_click(driver, hover_selector, click_selector,
                     hover_by=By.CSS_SELECTOR, click_by=By.CSS_SELECTOR,
                     timeout=settings.SMALL_TIMEOUT):
@@ -113,17 +123,17 @@ def hover_and_click(driver, hover_selector, click_selector,
     driver - the webdriver object (required)
     hover_selector - the css selector to hover over (required)
     click_selector - the css selector to click on (required)
-    hover_by - the method to search by (Default: By.CSS_SELECTOR)
-    click_by - the method to search by (Default: By.CSS_SELECTOR)
+    hover_by - the hover selector type to search by (Default: By.CSS_SELECTOR)
+    click_by - the click selector type to search by (Default: By.CSS_SELECTOR)
     timeout - number of seconds to wait for click element to appear after hover
     """
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     element = driver.find_element(by=hover_by, value=hover_selector)
     hover = ActionChains(driver).move_to_element(element)
-    hover.perform()
     for x in range(int(timeout * 10)):
         try:
+            hover.perform()
             element = driver.find_element(by=click_by, value=click_selector)
             element.click()
             return element
@@ -132,9 +142,12 @@ def hover_and_click(driver, hover_selector, click_selector,
             if now_ms >= stop_ms:
                 break
             time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
     raise NoSuchElementException(
-        "Element {%s} was not present after %s seconds!" %
-        (click_selector, timeout))
+        "Element {%s} was not present after %s second%s!" % (
+            click_selector, timeout, plural))
 
 
 def hover_element_and_click(driver, element, click_selector,
@@ -146,9 +159,9 @@ def hover_element_and_click(driver, element, click_selector,
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     hover = ActionChains(driver).move_to_element(element)
-    hover.perform()
     for x in range(int(timeout * 10)):
         try:
+            hover.perform()
             element = driver.find_element(by=click_by, value=click_selector)
             element.click()
             return element
@@ -157,9 +170,40 @@ def hover_element_and_click(driver, element, click_selector,
             if now_ms >= stop_ms:
                 break
             time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
     raise NoSuchElementException(
-        "Element {%s} was not present after %s seconds!" %
-        (click_selector, timeout))
+        "Element {%s} was not present after %s second%s!" % (
+            click_selector, timeout, plural))
+
+
+def hover_element_and_double_click(driver, element, click_selector,
+                                   click_by=By.CSS_SELECTOR,
+                                   timeout=settings.SMALL_TIMEOUT):
+    start_ms = time.time() * 1000.0
+    stop_ms = start_ms + (timeout * 1000.0)
+    hover = ActionChains(driver).move_to_element(element)
+    for x in range(int(timeout * 10)):
+        try:
+            hover.perform()
+            element_2 = driver.find_element(by=click_by, value=click_selector)
+            actions = ActionChains(driver)
+            actions.move_to_element(element_2)
+            actions.double_click(element_2)
+            actions.perform()
+            return element_2
+        except Exception:
+            now_ms = time.time() * 1000.0
+            if now_ms >= stop_ms:
+                break
+            time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
+    raise NoSuchElementException(
+        "Element {%s} was not present after %s second%s!" % (
+            click_selector, timeout, plural))
 
 
 def wait_for_element_present(driver, selector, by=By.CSS_SELECTOR,
@@ -171,17 +215,17 @@ def wait_for_element_present(driver, selector, by=By.CSS_SELECTOR,
     specified timeout.
     @Params
     driver - the webdriver object
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     timeout - the time to wait for elements in seconds
     @Returns
     A web element object
     """
-
     element = None
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             element = driver.find_element(by=by, value=selector)
             return element
@@ -190,10 +234,13 @@ def wait_for_element_present(driver, selector, by=By.CSS_SELECTOR,
             if now_ms >= stop_ms:
                 break
             time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
     if not element:
         raise NoSuchElementException(
-            "Element {%s} was not present after %s seconds!" % (
-                selector, timeout))
+            "Element {%s} was not present after %s second%s!" % (
+                selector, timeout, plural))
 
 
 def wait_for_element_visible(driver, selector, by=By.CSS_SELECTOR,
@@ -205,18 +252,18 @@ def wait_for_element_visible(driver, selector, by=By.CSS_SELECTOR,
     specified timeout.
     @Params
     driver - the webdriver object (required)
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     timeout - the time to wait for elements in seconds
 
     @Returns
     A web element object
     """
-
     element = None
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             element = driver.find_element(by=by, value=selector)
             if element.is_displayed():
@@ -252,25 +299,24 @@ def wait_for_text_visible(driver, text, selector, by=By.CSS_SELECTOR,
     @Params
     driver - the webdriver object (required)
     text - the text that is being searched for in the element (required)
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     timeout - the time to wait for elements in seconds
     @Returns
     A web element object that contains the text searched for
     """
-
     element = None
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             element = driver.find_element(by=by, value=selector)
-            if element.is_displayed():
-                if text in element.text:
-                    return element
-                else:
-                    element = None
-                    raise Exception()
+            if element.is_displayed() and text in element.text:
+                return element
+            else:
+                element = None
+                raise Exception()
         except Exception:
             now_ms = time.time() * 1000.0
             if now_ms >= stop_ms:
@@ -285,6 +331,49 @@ def wait_for_text_visible(driver, text, selector, by=By.CSS_SELECTOR,
             (text, selector, timeout, plural))
 
 
+def wait_for_exact_text_visible(driver, text, selector, by=By.CSS_SELECTOR,
+                                timeout=settings.LARGE_TIMEOUT):
+    """
+    Searches for the specified element by the given selector. Returns the
+    element object if the text matches exactly with the text in the element,
+    and the text is visible.
+    Raises an exception if the text or element do not appear
+    in the specified timeout.
+    @Params
+    driver - the webdriver object (required)
+    text - the exact text that is expected for the element (required)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
+    timeout - the time to wait for elements in seconds
+    @Returns
+    A web element object that contains the text searched for
+    """
+    element = None
+    start_ms = time.time() * 1000.0
+    stop_ms = start_ms + (timeout * 1000.0)
+    for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
+        try:
+            element = driver.find_element(by=by, value=selector)
+            if element.is_displayed() and text.strip() == element.text.strip():
+                return element
+            else:
+                element = None
+                raise Exception()
+        except Exception:
+            now_ms = time.time() * 1000.0
+            if now_ms >= stop_ms:
+                break
+            time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
+    if not element:
+        raise ElementNotVisibleException(
+            "Expected exact text {%s} for {%s} was not visible "
+            "after %s second%s!" % (text, selector, timeout, plural))
+
+
 def wait_for_element_absent(driver, selector, by=By.CSS_SELECTOR,
                             timeout=settings.LARGE_TIMEOUT):
     """
@@ -293,14 +382,14 @@ def wait_for_element_absent(driver, selector, by=By.CSS_SELECTOR,
     specified timeout.
     @Params
     driver - the webdriver object
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     timeout - the time to wait for elements in seconds
     """
-
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             driver.find_element(by=by, value=selector)
             now_ms = time.time() * 1000.0
@@ -324,14 +413,14 @@ def wait_for_element_not_visible(driver, selector, by=By.CSS_SELECTOR,
     specified timeout.
     @Params
     driver - the webdriver object (required)
-    selector - the locator that is used (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     timeout - the time to wait for the element in seconds
     """
-
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             element = driver.find_element(by=by, value=selector)
             if element.is_displayed():
@@ -351,17 +440,59 @@ def wait_for_element_not_visible(driver, selector, by=By.CSS_SELECTOR,
             selector, timeout, plural))
 
 
+def wait_for_text_not_visible(driver, text, selector, by=By.CSS_SELECTOR,
+                              timeout=settings.LARGE_TIMEOUT):
+    """
+    Searches for the text in the element of the given selector on the page.
+    Returns True if the text is not visible on the page within the timeout.
+    Raises an exception if the text is still present after the timeout.
+    @Params
+    driver - the webdriver object (required)
+    text - the text that is being searched for in the element (required)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
+    timeout - the time to wait for elements in seconds
+    @Returns
+    A web element object that contains the text searched for
+    """
+    start_ms = time.time() * 1000.0
+    stop_ms = start_ms + (timeout * 1000.0)
+    for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
+        if not is_text_visible(driver, text, selector, by=by):
+            return True
+        now_ms = time.time() * 1000.0
+        if now_ms >= stop_ms:
+            break
+        time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
+    raise Exception("Text {%s} in {%s} was still visible after %s "
+                    "second%s!" % (text, selector, timeout, plural))
+
+
 def find_visible_elements(driver, selector, by=By.CSS_SELECTOR):
     """
     Finds all WebElements that match a selector and are visible.
     Similar to webdriver.find_elements.
     @Params
     driver - the webdriver object (required)
-    selector - the locator that is used to search the DOM (required)
-    by - the method to search for the locator (Default: By.CSS_SELECTOR)
+    selector - the locator for identifying the page element (required)
+    by - the type of selector being used (Default: By.CSS_SELECTOR)
     """
     elements = driver.find_elements(by=by, value=selector)
-    return [element for element in elements if element.is_displayed()]
+    try:
+        v_elems = [element for element in elements if element.is_displayed()]
+        return v_elems
+    except (StaleElementReferenceException, ENI_Exception):
+        time.sleep(0.1)
+        elements = driver.find_elements(by=by, value=selector)
+        v_elems = []
+        for element in elements:
+            if element.is_displayed():
+                v_elems.append(element)
+        return v_elems
 
 
 def save_screenshot(driver, name, folder=None):
@@ -370,7 +501,7 @@ def save_screenshot(driver, name, folder=None):
     If the folder provided doesn't exist, it will get created.
     The screenshot will be in PNG format.
     """
-    if "." not in name:
+    if not name.endswith(".png"):
         name = name + ".png"
     if folder:
         abs_path = os.path.abspath('.')
@@ -381,12 +512,41 @@ def save_screenshot(driver, name, folder=None):
     else:
         screenshot_path = name
     try:
-        element = driver.find_element_by_tag_name('body')
+        element = driver.find_element(by=By.TAG_NAME, value="body")
         element_png = element.screenshot_as_png
         with open(screenshot_path, "wb") as file:
             file.write(element_png)
     except Exception:
-        driver.get_screenshot_as_file(screenshot_path)
+        if driver:
+            driver.get_screenshot_as_file(screenshot_path)
+        else:
+            pass
+
+
+def save_page_source(driver, name, folder=None):
+    """
+    Saves the page HTML to the current directory (or given subfolder).
+    If the folder specified doesn't exist, it will get created.
+    @Params
+    name - The file name to save the current page's HTML to.
+    folder - The folder to save the file to. (Default = current folder)
+    """
+    if not name.endswith(".html"):
+        name = name + ".html"
+    if folder:
+        abs_path = os.path.abspath('.')
+        file_path = abs_path + "/%s" % folder
+        if not os.path.exists(file_path):
+            os.makedirs(file_path)
+        html_file_path = "%s/%s" % (file_path, name)
+    else:
+        html_file_path = name
+    page_source = driver.page_source
+    html_file = codecs.open(html_file_path, "w+", "utf-8")
+    rendered_source = log_helper.get_html_source_with_base_href(
+        driver, page_source)
+    html_file.write(rendered_source)
+    html_file.close()
 
 
 def _get_last_page(driver):
@@ -460,10 +620,10 @@ def wait_for_and_switch_to_alert(driver, timeout=settings.LARGE_TIMEOUT):
     driver - the webdriver object (required)
     timeout - the time to wait for the alert in seconds
     """
-
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             alert = driver.switch_to.alert
             # Raises exception if no alert present
@@ -479,21 +639,34 @@ def wait_for_and_switch_to_alert(driver, timeout=settings.LARGE_TIMEOUT):
 
 def switch_to_frame(driver, frame, timeout=settings.SMALL_TIMEOUT):
     """
-    Wait for an iframe to appear, and switch to it. This should be usable
-    as a drop-in replacement for driver.switch_to.frame().
+    Wait for an iframe to appear, and switch to it. This should be
+    usable as a drop-in replacement for driver.switch_to.frame().
     @Params
     driver - the webdriver object (required)
-    frame - the frame element, name, or index
+    frame - the frame element, name, id, index, or selector
     timeout - the time to wait for the alert in seconds
     """
-
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     for x in range(int(timeout * 10)):
+        shared_utils.check_if_time_limit_exceeded()
         try:
             driver.switch_to.frame(frame)
             return True
         except NoSuchFrameException:
+            if type(frame) is str:
+                by = None
+                if page_utils.is_xpath_selector(frame):
+                    by = By.XPATH
+                else:
+                    by = By.CSS_SELECTOR
+                if is_element_visible(driver, frame, by=by):
+                    try:
+                        element = driver.find_element(by=by, value=frame)
+                        driver.switch_to.frame(element)
+                        return True
+                    except Exception:
+                        pass
             now_ms = time.time() * 1000.0
             if now_ms >= stop_ms:
                 break
@@ -510,11 +683,11 @@ def switch_to_window(driver, window, timeout=settings.SMALL_TIMEOUT):
     window - the window index or window handle
     timeout - the time to wait for the window in seconds
     """
-
     start_ms = time.time() * 1000.0
     stop_ms = start_ms + (timeout * 1000.0)
     if isinstance(window, int):
         for x in range(int(timeout * 10)):
+            shared_utils.check_if_time_limit_exceeded()
             try:
                 window_handle = driver.window_handles[window]
                 driver.switch_to.window(window_handle)
@@ -528,6 +701,7 @@ def switch_to_window(driver, window, timeout=settings.SMALL_TIMEOUT):
     else:
         window_handle = window
         for x in range(int(timeout * 10)):
+            shared_utils.check_if_time_limit_exceeded()
             try:
                 driver.switch_to.window(window_handle)
                 return True
