@@ -101,14 +101,12 @@ def _get_raw_css_from_xpath(xpath):
             elif match['mattr'] in ["text()", "."]:
                 attr = ":contains(^%s$)" % match['mvalue']
             elif match['mattr']:
-                if match["mvalue"].find(" ") != -1:
-                    match["mvalue"] = "\"%s\"" % match["mvalue"]
-                attr = "[%s=%s]" % (match['mattr'].replace("@", ""),
-                                    match['mvalue'])
+                attr = '[%s="%s"]' % (match['mattr'].replace("@", ""),
+                                      match['mvalue'])
         elif match['contained']:
             if match['cattr'].startswith("@"):
-                attr = "[%s*=%s]" % (match['cattr'].replace("@", ""),
-                                     match['cvalue'])
+                attr = '[%s*="%s"]' % (match['cattr'].replace("@", ""),
+                                       match['cvalue'])
             elif match['cattr'] == "text()":
                 attr = ":contains(%s)" % match['cvalue']
         else:
