@@ -9,42 +9,83 @@ from seleniumbase.core.style_sheet import style
 from seleniumbase.config import settings
 from seleniumbase.fixtures import js_utils
 
-LATEST_REPORT_DIR = settings.LATEST_REPORT_DIR
-ARCHIVE_DIR = settings.REPORT_ARCHIVE_DIR
-RESULTS_PAGE = settings.HTML_REPORT
-BAD_PAGE_LOG = settings.RESULTS_TABLE
-DEFAULT_VALIDATION_MESSAGE = settings.MASTERQA_DEFAULT_VALIDATION_MESSAGE
-WAIT_TIME_BEFORE_VERIFY = settings.MASTERQA_WAIT_TIME_BEFORE_VERIFY
-START_IN_FULL_SCREEN_MODE = settings.MASTERQA_START_IN_FULL_SCREEN_MODE
-MAX_IDLE_TIME_BEFORE_QUIT = settings.MASTERQA_MAX_IDLE_TIME_BEFORE_QUIT
-
 # This tool allows testers to quickly verify pages while assisted by automation
 
 
-class __MasterQATestCase__(BaseCase):
+class MasterQA(BaseCase):
 
-    def get_timestamp(self):
+    def setUp(self):
+        self.check_count = 0
+        self.auto_close_results_page = False
+        super(MasterQA, self).setUp(masterqa_mode=True)
+        self.LATEST_REPORT_DIR = settings.LATEST_REPORT_DIR
+        self.ARCHIVE_DIR = settings.REPORT_ARCHIVE_DIR
+        self.RESULTS_PAGE = settings.HTML_REPORT
+        self.BAD_PAGE_LOG = settings.RESULTS_TABLE
+        self.DEFAULT_VALIDATION_MESSAGE = (
+            settings.MASTERQA_DEFAULT_VALIDATION_MESSAGE)
+        self.WAIT_TIME_BEFORE_VERIFY = (
+            settings.MASTERQA_WAIT_TIME_BEFORE_VERIFY)
+        self.START_IN_FULL_SCREEN_MODE = (
+            settings.MASTERQA_START_IN_FULL_SCREEN_MODE)
+        self.MAX_IDLE_TIME_BEFORE_QUIT = (
+            settings.MASTERQA_MAX_IDLE_TIME_BEFORE_QUIT)
+        self.__manual_check_setup()
+        if self.headless:
+            self.auto_close_results_page = True
+        if self.START_IN_FULL_SCREEN_MODE:
+            self.maximize_window()
+
+    def verify(self, *args):
+        warn_msg = "\nWARNING: MasterQA skips manual checks in headless mode!"
+        self.check_count += 1
+        if self.headless:
+            if self.check_count == 1:
+                print(warn_msg)
+            return
+        # This is where the magic happens
+        self.__manual_page_check(*args)
+
+    def auto_close_results(self):
+        ''' If this method is called, the results page will automatically close
+        at the end of the test run, rather than waiting on the user to close
+        the results page manually.
+        '''
+        self.auto_close_results_page = True
+
+    def tearDown(self):
+        if self.headless and self.check_count > 0:
+            print("WARNING: %s manual checks were skipped!" % self.check_count)
+        if sys.exc_info()[1]:
+            self.__add_failure(sys.exc_info()[1])
+        self.__process_manual_check_results(self.auto_close_results_page)
+        super(MasterQA, self).tearDown()
+
+    ####################
+
+    def __get_timestamp(self):
         return str(int(time.time() * 1000))
 
-    def manual_check_setup(self):
+    def __manual_check_setup(self):
         self.manual_check_count = 0
         self.manual_check_successes = 0
         self.incomplete_runs = 0
         self.page_results_list = []
-        self.clear_out_old_logs(archive_past_runs=False)
+        self.__clear_out_old_logs(archive_past_runs=False)
 
-    def clear_out_old_logs(self, archive_past_runs=True, get_log_folder=False):
+    def __clear_out_old_logs(
+            self, archive_past_runs=True, get_log_folder=False):
         abs_path = os.path.abspath('.')
-        file_path = abs_path + "/%s" % LATEST_REPORT_DIR
+        file_path = abs_path + "/%s" % self.LATEST_REPORT_DIR
         if not os.path.exists(file_path):
             os.makedirs(file_path)
 
         if archive_past_runs:
             archive_timestamp = int(time.time())
-            if not os.path.exists("%s/../%s/" % (file_path, ARCHIVE_DIR)):
-                os.makedirs("%s/../%s/" % (file_path, ARCHIVE_DIR))
+            if not os.path.exists("%s/../%s/" % (file_path, self.ARCHIVE_DIR)):
+                os.makedirs("%s/../%s/" % (file_path, self.ARCHIVE_DIR))
             archive_dir = "%s/../%s/log_%s" % (
-                file_path, ARCHIVE_DIR, archive_timestamp)
+                file_path, self.ARCHIVE_DIR, archive_timestamp)
             shutil.move(file_path, archive_dir)
             os.makedirs(file_path)
             if get_log_folder:
@@ -52,13 +93,15 @@ class __MasterQATestCase__(BaseCase):
         else:
             # Just delete bad pages to make room for the latest run.
             filelist = [f for f in os.listdir(
-                "./%s" % LATEST_REPORT_DIR) if f.startswith("failed_") or (
-                f == RESULTS_PAGE) or (f.startswith("automation_failure")) or (
-                f == BAD_PAGE_LOG)]
+                "./%s" % self.LATEST_REPORT_DIR) if (
+                f.startswith("failed_")) or (
+                f == self.RESULTS_PAGE) or (
+                f.startswith("automation_failure")) or (
+                f == self.BAD_PAGE_LOG)]
             for f in filelist:
                 os.remove("%s/%s" % (file_path, f))
 
-    def jq_confirm_dialog(self, question):
+    def __jq_confirm_dialog(self, question):
         count = self.manual_check_count + 1
         title_content = ('<center><font color="#7700bb">Manual Check #%s:'
                          '</font></center><hr><font color="#0066ff">%s</font>'
@@ -97,9 +140,9 @@ class __MasterQATestCase__(BaseCase):
                 });""" % title_content)
         self.execute_script(jqcd)
 
-    def manual_page_check(self, *args):
+    def __manual_page_check(self, *args):
         if not args:
-            instructions = DEFAULT_VALIDATION_MESSAGE  # self.verify()
+            instructions = self.DEFAULT_VALIDATION_MESSAGE  # self.verify()
         else:
             instructions = str(args[0])
             if len(args) > 1:
@@ -111,6 +154,12 @@ class __MasterQATestCase__(BaseCase):
         elif instructions and "?" in instructions:
             question = instructions
 
+        wait_time_before_verify = self.WAIT_TIME_BEFORE_VERIFY
+        if self.verify_delay:
+            wait_time_before_verify = float(self.verify_delay)
+        # Allow a moment to see the full page before the dialog box pops up
+        time.sleep(wait_time_before_verify)
+
         use_jqc = False
         self.wait_for_ready_state_complete()
         if js_utils.is_jquery_confirm_activated(self.driver):
@@ -126,14 +175,8 @@ class __MasterQATestCase__(BaseCase):
                 use_jqc = False
 
         if use_jqc:
-            wait_time_before_verify = WAIT_TIME_BEFORE_VERIFY
-            if self.verify_delay:
-                wait_time_before_verify = float(self.verify_delay)
-            # Allow a moment to see the full page before the dialog box pops up
-            time.sleep(wait_time_before_verify)
-
             # Use the jquery_confirm library for manual page checks
-            self.jq_confirm_dialog(question)
+            self.__jq_confirm_dialog(question)
             time.sleep(0.02)
             waiting_for_response = True
             while waiting_for_response:
@@ -163,7 +206,7 @@ class __MasterQATestCase__(BaseCase):
                     {window.master_qa_result="Success!"}
                     else{window.master_qa_result="Failure!"}''' % question)
                 time.sleep(0.05)
-                self.wait_for_special_alert_absent()
+                self.__wait_for_special_alert_absent()
                 text = self.execute_script('return window.master_qa_result')
             else:
                 try:
@@ -175,7 +218,7 @@ class __MasterQATestCase__(BaseCase):
                     # Fix for https://github.com/mozilla/geckodriver/issues/431
                     pass
                 time.sleep(0.05)
-                self.wait_for_special_alert_absent()
+                self.__wait_for_special_alert_absent()
                 text = self.execute_script('return window.master_qa_result')
             status = text
 
@@ -193,13 +236,13 @@ class __MasterQATestCase__(BaseCase):
                     "-",
                     current_url,
                     self.browser,
-                    self.get_timestamp()[:-3],
+                    self.__get_timestamp()[:-3],
                     instructions,
                     "*"))
             return 1
         else:
             bad_page_name = "failed_check_%s.png" % self.manual_check_count
-            self.save_screenshot(bad_page_name, folder=LATEST_REPORT_DIR)
+            self.save_screenshot(bad_page_name, folder=self.LATEST_REPORT_DIR)
             self.page_results_list.append(
                 '"%s","%s","%s","%s","%s","%s","%s","%s"' % (
                     self.manual_check_count,
@@ -207,12 +250,13 @@ class __MasterQATestCase__(BaseCase):
                     bad_page_name,
                     current_url,
                     self.browser,
-                    self.get_timestamp()[:-3],
+                    self.__get_timestamp()[:-3],
                     instructions,
                     "*"))
             return 0
 
-    def wait_for_special_alert_absent(self, timeout=MAX_IDLE_TIME_BEFORE_QUIT):
+    def __wait_for_special_alert_absent(self):
+        timeout = self.MAX_IDLE_TIME_BEFORE_QUIT
         for x in range(int(timeout * 20)):
             try:
                 alert = self.driver.switch_to.alert
@@ -226,7 +270,7 @@ class __MasterQATestCase__(BaseCase):
         raise Exception(
             "%s seconds passed without human action! Stopping..." % timeout)
 
-    def add_failure(self, exception=None):
+    def __add_failure(self, exception=None):
         exc_info = None
         if exception:
             if hasattr(exception, 'msg'):
@@ -238,7 +282,7 @@ class __MasterQATestCase__(BaseCase):
 
         self.incomplete_runs += 1
         error_page = "automation_failure_%s.png" % self.incomplete_runs
-        self.save_screenshot(error_page, folder=LATEST_REPORT_DIR)
+        self.save_screenshot(error_page, folder=self.LATEST_REPORT_DIR)
         self.page_results_list.append(
             '"%s","%s","%s","%s","%s","%s","%s","%s"' % (
                 "ERR",
@@ -246,7 +290,7 @@ class __MasterQATestCase__(BaseCase):
                 error_page,
                 self.driver.current_url,
                 self.browser,
-                self.get_timestamp()[:-3],
+                self.__get_timestamp()[:-3],
                 "-",
                 exc_info))
         try:
@@ -257,10 +301,10 @@ class __MasterQATestCase__(BaseCase):
         except Exception:
             pass
 
-    def add_bad_page_log_file(self):
+    def __add_bad_page_log_file(self):
         abs_path = os.path.abspath('.')
-        file_path = abs_path + "/%s" % LATEST_REPORT_DIR
-        log_file = "%s/%s" % (file_path, BAD_PAGE_LOG)
+        file_path = abs_path + "/%s" % self.LATEST_REPORT_DIR
+        log_file = "%s/%s" % (file_path, self.BAD_PAGE_LOG)
         f = open(log_file, 'w')
         h_p1 = '''"Num","Result","Screenshot","URL","Browser","Epoch Time",'''
         h_p2 = '''"Verification Instructions","Additional Info"\n'''
@@ -270,17 +314,17 @@ class __MasterQATestCase__(BaseCase):
             f.write("%s\n" % line)
         f.close()
 
-    def add_results_page(self, html):
+    def __add_results_page(self, html):
         abs_path = os.path.abspath('.')
-        file_path = abs_path + "/%s" % LATEST_REPORT_DIR
-        results_file_name = RESULTS_PAGE
+        file_path = abs_path + "/%s" % self.LATEST_REPORT_DIR
+        results_file_name = self.RESULTS_PAGE
         results_file = "%s/%s" % (file_path, results_file_name)
         f = open(results_file, 'w')
         f.write(html)
         f.close()
         return results_file
 
-    def process_manual_check_results(self, auto_close_results_page=False):
+    def __process_manual_check_results(self, auto_close_results_page=False):
         perfection = True
         failures_count = self.manual_check_count - self.manual_check_successes
         print("\n\n*** Test Result: ***")
@@ -301,12 +345,12 @@ class __MasterQATestCase__(BaseCase):
                 print("WARNING: No manual checks were performed!")
         else:
             pass
-        self.add_bad_page_log_file()  # Includes successful results
+        self.__add_bad_page_log_file()  # Includes successful results
 
-        log_string = self.clear_out_old_logs(get_log_folder=True)
+        log_string = self.__clear_out_old_logs(get_log_folder=True)
         log_folder = log_string.split('/')[-1]
         abs_path = os.path.abspath('.')
-        file_path = abs_path + "/%s" % ARCHIVE_DIR
+        file_path = abs_path + "/%s" % self.ARCHIVE_DIR
         log_path = "%s/%s" % (file_path, log_folder)
         web_log_path = "file://%s" % log_path
 
@@ -337,9 +381,9 @@ class __MasterQATestCase__(BaseCase):
                      %s</h1>''' % summary_table
 
         log_link_shown = '../%s%s/' % (
-            ARCHIVE_DIR, web_log_path.split(ARCHIVE_DIR)[1])
-        csv_link = '%s/%s' % (web_log_path, BAD_PAGE_LOG)
-        csv_link_shown = '%s' % BAD_PAGE_LOG
+            self.ARCHIVE_DIR, web_log_path.split(self.ARCHIVE_DIR)[1])
+        csv_link = '%s/%s' % (web_log_path, self.BAD_PAGE_LOG)
+        csv_link_shown = '%s' % self.BAD_PAGE_LOG
         log_table = '''<p><p><p><p><h2><table><tbody>
             <tr><td>LOG FILES LINK:&nbsp;&nbsp;<td><a href="%s">%s</a></tr>
             <tr><td>RESULTS TABLE:&nbsp;&nbsp;<td><a href="%s">%s</a></tr>
@@ -371,8 +415,8 @@ class __MasterQATestCase__(BaseCase):
             summary_table, log_table, failure_table)
         report_html = '<html><head>%s</head><body>%s</body></html>' % (
             style, table_view)
-        results_file = self.add_results_page(report_html)
-        archived_results_file = log_path + '/' + RESULTS_PAGE
+        results_file = self.__add_results_page(report_html)
+        archived_results_file = log_path + '/' + self.RESULTS_PAGE
         shutil.copyfile(results_file, archived_results_file)
         print(
             "\n*** The results html page is located at: ***\n" + results_file)
@@ -385,41 +429,3 @@ class __MasterQATestCase__(BaseCase):
             print("\n*** Close the html report window to continue ***")
             while len(self.driver.window_handles):
                 time.sleep(0.1)
-
-
-class MasterQA(__MasterQATestCase__):
-
-    def setUp(self):
-        self.check_count = 0
-        self.auto_close_results_page = False
-        super(__MasterQATestCase__, self).setUp()
-        self.manual_check_setup()
-        if self.headless:
-            self.auto_close_results_page = True
-        if START_IN_FULL_SCREEN_MODE:
-            self.maximize_window()
-
-    def verify(self, *args):
-        warn_msg = "\nWARNING: MasterQA skips manual checks in headless mode!"
-        self.check_count += 1
-        if self.headless:
-            if self.check_count == 1:
-                print(warn_msg)
-            return
-        # This is where the magic happens
-        self.manual_page_check(*args)
-
-    def auto_close_results(self):
-        ''' If this method is called, the results page will automatically close
-        at the end of the test run, rather than waiting on the user to close
-        the results page manually.
-        '''
-        self.auto_close_results_page = True
-
-    def tearDown(self):
-        if self.headless and self.check_count > 0:
-            print("WARNING: %s manual checks were skipped!" % self.check_count)
-        if sys.exc_info()[1]:
-            self.add_failure(sys.exc_info()[1])
-        self.process_manual_check_results(self.auto_close_results_page)
-        super(__MasterQATestCase__, self).tearDown()
