@@ -1,11 +1,6 @@
-"""
-This is the Nose plugin for saving logs and setting a test environment.
-Vars include "env" and "log_path".
-You can have tests behave differently based on the environment.
-You can access the values of these variables from the tests.
-"""
+# -*- coding: utf-8 -*-
+""" This is the Nose plugin for setting a test environment and saving logs. """
 
-import os
 import sys
 import time
 from nose.plugins import Plugin
@@ -17,25 +12,27 @@ from seleniumbase.fixtures import constants, errors
 
 class Base(Plugin):
     """
-    The base_plugin includes the following variables for nosetest runs:
-    self.env -- The environment for the tests to use (Usage: --env=ENV)
-    self.data -- Any extra data to pass to the tests (Usage: --data=DATA)
-    self.log_path -- The directory where log files get saved to
-                     (Usage: --log_path=LOG_PATH)
-    self.report -- The option to create a fancy report after tests complete
-                   (Usage: --report)
-    self.show_report -- If self.report is turned on, then the report will
-                        display immediately after tests complete their run.
-                        Only use this when running tests locally, as this will
-                        pause the test run until the report window is closed.
-                        (Usage: --show_report)
+    This parser plugin includes the following command-line options for Nose:
+    --env=ENV  (Set a test environment. Use "self.env" to use this in tests.)
+    --data=DATA  (Extra data to pass to tests. Use "self.data" in tests.)
+    --var1=DATA  (Extra data to pass to tests. Use "self.var1" in tests.)
+    --var2=DATA  (Extra data to pass to tests. Use "self.var2" in tests.)
+    --var3=DATA  (Extra data to pass to tests. Use "self.var3" in tests.)
+    --settings-file=FILE  (Overrides SeleniumBase settings.py values.)
+    --archive-logs  (Archive old log files instead of deleting them.)
+    --report  (The option to create a fancy report after tests complete.)
+    --show-report   If self.report is turned on, then the report will
+                    display immediately after tests complete their run.
+                    Only use this when running tests locally, as this will
+                    pause the test run until the report window is closed.
     """
-    name = 'testing_base'  # Usage: --with-testing_base
+    name = 'testing_base'  # Usage: --with-testing_base  (Enabled by default)
 
     def options(self, parser, env):
         super(Base, self).options(parser, env=env)
         parser.add_option(
-            '--env', action='store',
+            '--env',
+            action='store',
             dest='environment',
             choices=(
                 constants.Environment.QA,
@@ -48,32 +45,78 @@ class Base(Plugin):
             default=constants.Environment.TEST,
             help="The environment to run the tests in.")
         parser.add_option(
-            '--data', dest='data',
+            '--data',
+            dest='data',
             default=None,
-            help='Extra data to pass from the command line.')
+            help='Extra data to pass to tests from the command line.')
         parser.add_option(
-            '--log_path', dest='log_path',
+            '--var1',
+            dest='var1',
+            default=None,
+            help='Extra data to pass to tests from the command line.')
+        parser.add_option(
+            '--var2',
+            dest='var2',
+            default=None,
+            help='Extra data to pass to tests from the command line.')
+        parser.add_option(
+            '--var3',
+            dest='var3',
+            default=None,
+            help='Extra data to pass to tests from the command line.')
+        parser.add_option(
+            '--settings_file', '--settings-file', '--settings',
+            action='store',
+            dest='settings_file',
+            default=None,
+            help="""The file that stores key/value pairs for overriding
+                    values in the SeleniumBase settings.py file.""")
+        parser.add_option(
+            '--log_path', '--log-path',
+            dest='log_path',
             default='latest_logs/',
-            help='Where the log files are saved.')
+            help='Where the log files are saved. (No longer editable!)')
         parser.add_option(
-            '--report', action="store_true", dest='report',
+            '--archive_logs', '--archive-logs',
+            action="store_true",
+            dest='archive_logs',
+            default=False,
+            help="Archive old log files instead of deleting them.")
+        parser.add_option(
+            '--report',
+            action="store_true",
+            dest='report',
             default=False,
             help='Create a fancy report at the end of the test suite.')
         parser.add_option(
-            '--show_report', action="store_true",
+            '--show_report', '--show-report',
+            action="store_true",
             dest='show_report',
             default=False,
             help="If true when using report, will display it after tests run.")
         found_processes_arg = False
+        found_timeout_arg = False
         for arg in sys.argv:
             if "--processes=" in arg:
                 found_processes_arg = True
+            if "--timeout=" in arg:
+                found_timeout_arg = True
         if found_processes_arg:
             print("* WARNING: Don't use multi-threading with nosetests! *")
             parser.add_option(
-                '--processes', dest='processes',
+                '--processes',
+                dest='processes',
                 default=0,
                 help="WARNING: Don't use multi-threading with nosetests!")
+        if found_timeout_arg:
+            print("\n  WARNING: Don't use --timeout=s from pytest-timeout!")
+            print("  It's not thread-safe for WebDriver processes!")
+            print("  Use --time-limit=s from SeleniumBase instead!\n")
+            parser.add_option(
+                '--timeout',
+                dest='timeout',
+                default=0,
+                help="Don't use --timeout=s! Use --time-limit=s instead!")
 
     def configure(self, options, conf):
         super(Base, self).configure(options, conf)
@@ -88,23 +131,29 @@ class Base(Plugin):
         self.page_results_list = []
         self.test_count = 0
         self.import_error = False
-        log_path = options.log_path
-        log_helper.log_folder_setup(log_path)
+        log_path = 'latest_logs/'
+        archive_logs = options.archive_logs
+        log_helper.log_folder_setup(log_path, archive_logs)
         if self.report_on:
             report_helper.clear_out_old_report_logs(archive_past_runs=False)
 
     def beforeTest(self, test):
-        test_logpath = self.options.log_path + "/" + test.id()
-        if not os.path.exists(test_logpath):
-            os.makedirs(test_logpath)
         test.test.environment = self.options.environment
         test.test.env = self.options.environment  # Add a shortened version
         test.test.data = self.options.data
+        test.test.var1 = self.options.var1
+        test.test.var2 = self.options.var2
+        test.test.var3 = self.options.var3
+        test.test.settings_file = self.options.settings_file
+        test.test.log_path = self.options.log_path
         test.test.args = self.options
+        test.test.report_on = self.report_on
         self.test_count += 1
         self.start_time = float(time.time())
 
     def finalize(self, result):
+        log_helper.archive_logs_if_set(
+            self.options.log_path, self.options.archive_logs)
         if self.report_on:
             if not self.import_error:
                 report_helper.add_bad_page_log_file(self.page_results_list)
