@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 Converts a Selenium IDE recording that was exported as a Python WebDriver
 unittest file into SeleniumBase Python file.
@@ -50,9 +51,8 @@ def main():
     uses_keys = False
     uses_select = False
 
-    f = open(webdriver_python_file, 'r')
-    all_code = f.read()
-    f.close()
+    with open(webdriver_python_file, 'r', encoding='utf-8') as f:
+        all_code = f.read()
     if "def test_" not in all_code:
         raise Exception("\n\n`%s` is not a valid Python unittest.TestCase "
                         "file!\n\nExpecting: %s\n\n"
@@ -151,9 +151,29 @@ def main():
             r'''\.click\(\)\s*$''', line)
         if data:
             whitespace = data.group(1)
-            selector = '#%s' % data.group(2)
+            selector = '#%s' % data.group(2).replace('#', '\\#')
             selector = selector.replace('[', '\\[').replace(']', '\\]')
-            command = '''%sself.click('%s')''' % (whitespace, selector)
+            selector = selector.replace('.', '\\.')
+            raw = ""
+            if "\\[" in selector or "\\]" in selector or "\\." in selector:
+                raw = "r"
+            command = '''%sself.click(%s'%s')''' % (whitespace, raw, selector)
+            seleniumbase_lines.append(command)
+            continue
+
+        # Handle .find_element_by_id() + .submit()
+        data = re.match(
+            r'''^(\s*)driver\.find_element_by_id\(\"(\S+)\"\)'''
+            r'''\.submit\(\)\s*$''', line)
+        if data:
+            whitespace = data.group(1)
+            selector = '#%s' % data.group(2).replace('#', '\\#')
+            selector = selector.replace('[', '\\[').replace(']', '\\]')
+            selector = selector.replace('.', '\\.')
+            raw = ""
+            if "\\[" in selector or "\\]" in selector or "\\." in selector:
+                raw = "r"
+            command = '''%sself.submit(%s'%s')''' % (whitespace, raw, selector)
             seleniumbase_lines.append(command)
             continue
 
@@ -163,11 +183,15 @@ def main():
             r'''\.send_keys\(\"([\S\s]+)\"\)\s*$''', line)
         if data:
             whitespace = data.group(1)
-            selector = '#%s' % data.group(2)
+            selector = '#%s' % data.group(2).replace('#', '\\#')
             selector = selector.replace('[', '\\[').replace(']', '\\]')
+            selector = selector.replace('.', '\\.')
+            raw = ""
+            if "\\[" in selector or "\\]" in selector or "\\." in selector:
+                raw = "r"
             text = data.group(3)
-            command = '''%sself.update_text('%s', '%s')''' % (
-                whitespace, selector, text)
+            command = '''%sself.update_text(%s'%s', '%s')''' % (
+                whitespace, raw, selector, text)
             seleniumbase_lines.append(command)
             continue
 
@@ -178,11 +202,15 @@ def main():
         if data:
             uses_keys = True
             whitespace = data.group(1)
-            selector = '#%s' % data.group(2)
+            selector = '#%s' % data.group(2).replace('#', '\\#')
             selector = selector.replace('[', '\\[').replace(']', '\\]')
+            selector = selector.replace('.', '\\.')
+            raw = ""
+            if "\\[" in selector or "\\]" in selector or "\\." in selector:
+                raw = "r"
             key = 'Keys.%s' % data.group(3)
-            command = '''%sself.send_keys('%s', %s)''' % (
-                whitespace, selector, key)
+            command = '''%sself.send_keys(%s'%s', %s)''' % (
+                whitespace, raw, selector, key)
             seleniumbase_lines.append(command)
             continue
 
@@ -197,6 +225,17 @@ def main():
             seleniumbase_lines.append(command)
             continue
 
+        # Handle .find_element_by_name() + .submit()
+        data = re.match(
+            r'''^(\s*)driver\.find_element_by_name\(\"(\S+)\"\)'''
+            r'''\.submit\(\)\s*$''', line)
+        if data:
+            whitespace = data.group(1)
+            selector = '[name="%s"]' % data.group(2)
+            command = '''%sself.submit('%s')''' % (whitespace, selector)
+            seleniumbase_lines.append(command)
+            continue
+
         # Handle .find_element_by_name() + .send_keys()
         data = re.match(
             r'''^(\s*)driver\.find_element_by_name\(\"(\S+)\"\)'''
@@ -237,6 +276,19 @@ def main():
             seleniumbase_lines.append(command)
             continue
 
+        # Handle .find_element_by_css_selector() + .submit()
+        data = re.match(
+            r'''^(\s*)driver\.find_element_by_css_selector\(\"([\S\s]+)\"\)'''
+            r'''\.submit\(\)\s*$''', line)
+        if data:
+            whitespace = data.group(1)
+            selector = '%s' % data.group(2)
+            command = '''%sself.submit('%s')''' % (whitespace, selector)
+            if command.count('\\"') == command.count('"'):
+                command = command.replace('\\"', '"')
+            seleniumbase_lines.append(command)
+            continue
+
         # Handle .find_element_by_css_selector() + .send_keys()
         data = re.match(
             r'''^(\s*)driver\.find_element_by_css_selector\(\"([\S\s]+)\"\)'''
@@ -308,7 +360,7 @@ def main():
             whitespace = data.group(1)
             selector = '%s' % data.group(2)
             visible_text = '%s' % data.group(3)
-            command = '''%sself.pick_select_option_by_text('%s', '%s')''' % (
+            command = '''%sself.select_option_by_text('%s', '%s')''' % (
                 whitespace, selector, visible_text)
             if command.count('\\"') == command.count('"'):
                 command = command.replace('\\"', '"')
@@ -322,10 +374,15 @@ def main():
             r'''\"([\S\s]+)\"\)\s*$''', line)
         if data:
             whitespace = data.group(1)
-            selector = '#%s' % data.group(2)
+            selector = '#%s' % data.group(2).replace('#', '\\#')
+            selector = selector.replace('[', '\\[').replace(']', '\\]')
+            selector = selector.replace('.', '\\.')
+            raw = ""
+            if "\\[" in selector or "\\]" in selector or "\\." in selector:
+                raw = "r"
             visible_text = '%s' % data.group(3)
-            command = '''%sself.pick_select_option_by_text('%s', '%s')''' % (
-                whitespace, selector, visible_text)
+            command = '''%sself.select_option_by_text(%s'%s', '%s')''' % (
+                whitespace, raw, selector, visible_text)
             if command.count('\\"') == command.count('"'):
                 command = command.replace('\\"', '"')
             seleniumbase_lines.append(command)
@@ -340,7 +397,7 @@ def main():
             whitespace = data.group(1)
             selector = '%s' % data.group(2)
             visible_text = '%s' % data.group(3)
-            command = '''%sself.pick_select_option_by_text("%s", '%s')''' % (
+            command = '''%sself.select_option_by_text("%s", '%s')''' % (
                 whitespace, selector, visible_text)
             if command.count('\\"') == command.count('"'):
                 command = command.replace('\\"', '"')
@@ -356,7 +413,7 @@ def main():
             whitespace = data.group(1)
             selector = '[name="%s"]' % data.group(2)
             visible_text = '%s' % data.group(3)
-            command = '''%sself.pick_select_option_by_text('%s', '%s')''' % (
+            command = '''%sself.select_option_by_text('%s', '%s')''' % (
                 whitespace, selector, visible_text)
             if command.count('\\"') == command.count('"'):
                 command = command.replace('\\"', '"')
@@ -370,6 +427,14 @@ def main():
         if data:
             whitespace = data.group(1)
             xpath = '%s' % data.group(2)
+            if './/*[normalize-space(text())' in xpath and (
+                    "normalize-space(.)='" in xpath):
+                x_match = re.match(
+                    r'''^[\S\s]+normalize-'''
+                    r'''space\(\.\)=\'([\S\s]+)\'\]\)[\S\s]+''', xpath)
+                if x_match:
+                    partial_link_text = x_match.group(1)
+                    xpath = "partial_link=%s" % partial_link_text
             uni = ""
             if '(u"' in line:
                 uni = "u"
@@ -379,6 +444,22 @@ def main():
             seleniumbase_lines.append(command)
             continue
 
+        # Handle .find_element_by_xpath() + .submit()
+        data = re.match(
+            r'''^(\s*)driver\.find_element_by_xpath\(u?\"([\S\s]+)\"\)'''
+            r'''\.submit\(\)\s*$''', line)
+        if data:
+            whitespace = data.group(1)
+            xpath = '%s' % data.group(2)
+            uni = ""
+            if '(u"' in line:
+                uni = "u"
+                has_unicode = True
+            command = '''%sself.submit(%s"%s")''' % (
+                whitespace, uni, xpath)
+            seleniumbase_lines.append(command)
+            continue
+
         # Handle .find_element_by_link_text() + .click()
         data = re.match(
             r'''^(\s*)driver\.find_element_by_link_text\(u?\"([\S\s]+)\"\)'''
@@ -641,7 +722,7 @@ def main():
     # Create SeleniumBase test file
     base_file_name = webdriver_python_file.split('.py')[0]
     converted_file_name = base_file_name + "_SB.py"
-    out_file = codecs.open(converted_file_name, "w+")
+    out_file = codecs.open(converted_file_name, "w+", encoding='utf-8')
     out_file.writelines(seleniumbase_code)
     out_file.close()
     print('\n>>> [%s] was created from [%s]\n' % (
